#ifndef Lib_BaseSocket_h
#define Lib_BaseSocket_h

#include <string>
#include <vector>

#include <Lib/typedefs.h>

#include <ostream>

namespace Lib {

class BaseSocket { 
public:
  virtual bool accept() = 0;
  virtual bool connect() = 0;
  virtual void disconnect() = 0;
public:        
  BaseSocket(std::ostream&,int = 0);  
  BaseSocket(const BaseSocket&);  
  virtual ~BaseSocket();
  BaseSocket& operator=(const BaseSocket&);  
public:        
  bool isConnected() const;
  int socket() const;
public:
  bool send_buffer(const char*,uint64); 
  bool wait_buffer(char*,uint64); 
  bool send_uint64(uint64); 
  bool wait_uint64(uint64&); 
  bool send_string(const std::string&); 
  bool wait_string(std::string&); 
  bool send_text(const std::vector<std::string>&,bool=false); 
  bool wait_text(std::vector<std::string>&,bool=false); 
  bool send_file(const std::string&); 
  bool wait_file(const std::string&); 
protected:        
  void closeSocket();
protected:        
  std::ostream& m_out;
  int fVerboseLevel;
  int fSocket;
  bool fIsLE;
};

}

#endif

